package cc.fascinated.bat.features.autorole.profile;

import cc.fascinated.bat.common.Profile;
import cc.fascinated.bat.service.DiscordService;
import lombok.Getter;
import lombok.Setter;
import net.dv8tion.jda.api.entities.Role;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Setter @Getter
public class AutoRoleProfile extends Profile {
    private static final int DEFAULT_MAX_ROLES = 10;

    /**
     * The roles to assign when a user joins
     */
    private List<String> roleIds;

    /**
     * The maximum amount of roles that can be set
     */
    private int maxRoles = 10;

    public AutoRoleProfile() {
        super("auto-role");
    }

    /**
     * Gets the amount of role slots left
     *
     * @return the amount
     */
    public int getRoleSlotsLeft() {
        return maxRoles - getRoles().size();
    }

    /**
     * Gets the amount of role slots in use
     *
     * @return the amount
     */
    public int getRoleSlotsInUse() {
        return getRoles().size();
    }

    /**
     * Adds a role to assign when a user joins
     *
     * @param roleId the role ID to add
     */
    public void addRole(String roleId) {
        if (this.roleIds == null) {
            this.roleIds = new ArrayList<>();
        }
        this.roleIds.add(roleId);
    }

    /**
     * Removes a role from being assigned when a user joins
     *
     * @param roleId the role ID to remove
     */
    public void removeRole(String roleId) {
        if (this.roleIds == null) {
            this.roleIds = new ArrayList<>();
        }
        this.roleIds.remove(roleId);
    }

    /**
     * Checks if a role is set to be assigned when a user joins
     *
     * @param roleId the role ID to check
     * @return whether the role is set
     */
    public boolean hasRole(String roleId) {
        if (this.roleIds == null) {
            this.roleIds = new ArrayList<>();
        }
        return this.roleIds.contains(roleId);
    }

    /**
     * Gets the roles to assign when a user joins
     *
     * @return the roles
     */
    public List<Role> getRoles() {
        if (this.roleIds == null) {
            this.roleIds = new ArrayList<>();
        }
        List<Role> roles = new ArrayList<>();
        for (String id : this.roleIds) {
            Role role = DiscordService.JDA.getRoleById(id);
            if (role != null) {
                roles.add(role);
            }
        }
        return roles;
    }

    @Override
    public void reset() {
        roleIds.clear();
        maxRoles = DEFAULT_MAX_ROLES;
    }
}
