package cc.fascinated.bat.features.birthday.command;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.birthday.profile.BirthdayProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.GuildService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;


/**
 * @author Fascinated (fascinated7)
 */
@Component("birthday:add.sub")
public class SetSubCommand extends BatSubCommand {
    private static final SimpleDateFormat FORMATTER = new SimpleDateFormat("dd/MM/yyyy");
    private final GuildService guildService;

    @Autowired
    public SetSubCommand(GuildService guildService) {
        super("set", "Add your birthday to this guild");
        super.addOption(OptionType.STRING, "birthday", "Your birthday (format: DAY/MONTH/YEAR - 01/05/2004)", true);
        this.guildService = guildService;
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction interaction) {
        BirthdayProfile profile = guild.getProfile(BirthdayProfile.class);

        if (!profile.hasChannelSetup()) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("Birthdays have not been enabled in this guild. Please ask an administrator to enable them.")
                    .build()).queue();
            return;
        }

        OptionMapping birthdayOption = interaction.getOption("birthday");
        if (birthdayOption == null) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("You must provide a birthday")
                    .build()).queue();
            return;
        }
        String birthdayString = birthdayOption.getAsString();
        Date birthday = parseBirthday(birthdayString);
        if (birthday == null) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("Invalid birthday format. Please use the format: DAY/MONTH/YEAR - 01/05/2004")
                    .build()).queue();
            return;
        }

        profile.addBirthday(member.getId(), birthday);
        guildService.saveGuild(guild);

        interaction.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("You have updated your birthday!")
                .build()).queue();
    }

    /**
     * Parses a birthday from the string
     *
     * @param birthday the date to parse
     * @return the birthday
     */
    private Date parseBirthday(String birthday) {
        try {
            return FORMATTER.parse(birthday);
        } catch (ParseException ignored) {}
        return null;
    }
}
