package cc.fascinated.bat.features.scoresaber;

import cc.fascinated.bat.common.DateUtils;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.NumberUtils;
import cc.fascinated.bat.common.ScoreSaberUtils;
import cc.fascinated.bat.features.Feature;
import cc.fascinated.bat.features.scoresaber.command.numberone.NumberOneFeedCommand;
import cc.fascinated.bat.features.scoresaber.command.scoresaber.ScoreSaberCommand;
import cc.fascinated.bat.features.scoresaber.command.userfeed.UserFeedCommand;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberLeaderboardToken;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberPlayerScoreToken;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberScoreToken;
import cc.fascinated.bat.service.CommandService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.MessageEmbed;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
public class ScoreSaberFeature extends Feature {
    @Autowired
    public ScoreSaberFeature(@NonNull ApplicationContext context, @NonNull CommandService commandService) {
        super("ScoreSaber", Category.BEAT_SABER);

        commandService.registerCommand(context.getBean(ScoreSaberCommand.class));
        commandService.registerCommand(context.getBean(UserFeedCommand.class));
        commandService.registerCommand(context.getBean(NumberOneFeedCommand.class));
    }

    /**
     * Builds an embed for a score.
     *
     * @param score The score.
     * @return The embed.
     */
    public static MessageEmbed buildScoreEmbed(ScoreSaberPlayerScoreToken score) {
        ScoreSaberScoreToken scoreToken = score.getScore();
        ScoreSaberLeaderboardToken leaderboardToken = score.getLeaderboard();
        ScoreSaberScoreToken.LeaderboardPlayerInfo playerInfo = scoreToken.getLeaderboardPlayerInfo();

        String thumbnailUrl = String.format("https://cdn.scoresaber.com/covers/%s.png", leaderboardToken.getSongHash());
        String authorUrl = String.format("https://scoresaber.com/u/%s", playerInfo.getId());
        String description = String.format("**%s** (%s%s)\n[[Map Link]](%s) [[SS Profile]](%s)",
                leaderboardToken.getSongName(),
                ScoreSaberUtils.getFormattedDifficulty(leaderboardToken.getDifficulty().getDifficulty()),
                leaderboardToken.isRanked() ? " " + leaderboardToken.getStars() + " ⭐" : "",
                String.format("https://scoresaber.com/leaderboard/%s", leaderboardToken.getId()),
                authorUrl
        );

        String accuracy = leaderboardToken.getMaxScore() == 0 ? "N/A" :
                String.format("%s%%", NumberUtils.formatNumberCommas(((double) scoreToken.getBaseScore() / leaderboardToken.getMaxScore()) * 100));

        String rawPp = scoreToken.getPp() == 0 ? "Unranked" : NumberUtils.formatNumberCommas(scoreToken.getPp());
        String rank = String.format("#%s", NumberUtils.formatNumberCommas(scoreToken.getRank()));
        String misses = String.format("%s", scoreToken.getMissedNotes());
        String badCuts = String.format("%s", scoreToken.getBadCuts());
        String maxCombo = String.format("%s %s",
                scoreToken.getMaxCombo(),
                scoreToken.getMaxCombo() == leaderboardToken.getMaxScore() ? "(FC)" : ""
        );

        return EmbedUtils.genericEmbed()
                .setThumbnail(thumbnailUrl)
                .setAuthor(playerInfo.getName() + " just set a new score!", authorUrl, playerInfo.getProfilePicture())
                .setDescription(description)
                .addField("Accuracy", accuracy, true)
                .addField("Raw PP", rawPp, true)
                .addField("Rank", rank, true)
                .addField("Misses", misses, true)
                .addField("Bad Cuts", badCuts, true)
                .addField("Max Combo", maxCombo, true)
                .setTimestamp(DateUtils.getDateFromString(scoreToken.getTimeSet()).toInstant())
                .build();
    }
}
